import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:flutterx_material_tool/flutterx_material_tool.dart';
import 'package:flutterx_material_tool/src/model/colors.dart';
import 'package:flutterx_material_tool/src/model/material_palette.dart';
import 'package:flutterx_material_tool/src/tool.dart';

void main() {
  test('Test tool', () {
    final palette = MaterialPalette.primary;
    final swatch = generateMaterialPalette(RGBColor.fromColor(const Color(0xff607D8B)), palette);
    swatch.printDescription(palette.shades);
    for (final primary in palette.palettes) {
      final swatch = generateMaterialPalette(
          RGBColor.fromColor(primary[palette.primaryIndex].toLCHColor().toRGBColor().toColor()), palette);
      print('---');
      for (final shade in palette.shades) {
        final description = '${shade.toString().padLeft(3, '0')}: ${swatch[shade]!.hex}';
        print(
            '    $description ${swatch[shade] == primary[palette.shades.indexOf(shade)].toLCHColor().toRGBColor().toColor() ? 'Y' : 'X'}');
      }
    }
  });
}

extension ColorExt on Color {
  String get hex => '#${value.toRadixString(16).substring(2).toUpperCase()}';
}

extension ColorSwatchExt on ColorSwatch<int> {
  void printDescription(List<int> shades, [String? label]) {
    var indent = '';
    if (label != null) {
      print(label);
      indent = '    ';
    }
    for (final shade in shades) {
      final description = '${shade.toString().padLeft(3, '0')}: ${this[shade]!.hex}';
      print('$indent$description');
    }
  }
}
